// version 1.0
#include "pins.h"       // mcu pin assignment
#include "netparms.h"   // network parameters
#include "get.h"        // GET request and search keys

/*********     WeatherWiz for TI LaunchPad     *********/

// W5200 commands
static const char SEND_CMD = 0x20;
static const char OPEN_CMD = 0x01;
static const char CONN_CMD = 0x04;
static const char CLOSE_CMD = 0x08;
#define TCP_MODE OPEN_CMD  // both are 0x01

// W5200 registers
#define MR 0            // W5200 mode
#define S0_MR 0x4000    // socket0 mode
#define S0_CR 0x4001    // .. command
#define S0_SR 0x4003    // .. status
#define S0_PORT 0x4004  // .. source port
#define S0_DIPR 0x400c  // .. dest ip
#define S0_RXMSZ 0x401e // .. rxbuffer size
#define S0_TXWR 0x4024  // .. txbuffer write ptr
#define S0_RXRSR 0x4026 // .. receive size
#define RXBASE 0xc000   // rxbuffer address
#define TXBASE 0x8000   // txbuffer address
#define TXMASK 0x7ff    // txbuffer size = 2KB
#define XMLS   0x8800   // use up to 14KB of txbuffer to store xml data

// global variables (ram)
unsigned int a, pa, c, tc, i, j, k, m, n; // logical/physical addr, count, total, counters
char cap, ok, data;  // capture & ok flags, Wiz data

main(void){
  Init();                          // init hardware
  while (1) {
    pin_low(PWDN);                 // power up Wiz820io
    wait_secs(1);
    conout_str("\rWeatherWiz for ");
    conout_str(platform);
    Issueget();
    if (ok) {
      Searchweatherpage();
      Displayweatherdata();
    }
    pin_high(PWDN);               // power down Wiz820io
    wait_secs(3600);              // weather updates hourly
  }
}

Init(void){
// init platform specific hardware and Wiz820io MISO,MOSI,SCLK
  halinit();
 // init other Wiz820io pins
  pin_high(nSS);
  dir_out(nSS);
  pin_low(PWDN);
  dir_out(PWDN);
// reset wiz820io
  pin_low(nRESET);
  dir_out(nRESET);
  wait_secs(1);
  pin_high(nRESET);
  wait_secs(1);
// init ip addresses
  Wizwrite(MR, 19, &srcip);     // our (client) ip info
  Wizwrite(S0_DIPR, 8, &dstip); // dest ip (server) info
// W5200 socket buffer configuration
// socket 0-7 txbuffer size is 2KB (reset default)
// socket 0 rxbuffer size set to 16KB, socket 1-7 0KB
  data = 0x10;
  a = S0_RXMSZ;
  Wizwrite(a, 1, &data);     // socket 0 is 16KB
  a = (a + 0x100);
  data = 0;
  for (i=0; i<7; i++) {      // sockets 1-7 0KB
    Wizwrite(a, 1, &data);
    a = (a + 0x100);
  }
}

Issueget(void){  // connect, issue GET request, receive data, disconnect
// connect
  led_on();
  ok=1;                             // ok so far
  Wizread_a(S0_PORT);
  if (a++ < 1024) a = 1024;         // don't use 'well known' ports 0-1023
  Wizwrite_a(S0_PORT);
  Wizwrite(S0_MR, 1, &TCP_MODE);    // open connection
  Wizwrite(S0_CR, 1, &OPEN_CMD);
  do {
    Wizread(S0_SR, 1, &data);
    if (data == 0) {
      ok = 0;                       // socket did not open
      return;
    }
  } while (data != 0x13);           // opened
  conout_str("\rconnecting -");
  Wizwrite(S0_CR, 1, &CONN_CMD);    // connect
  do {
    Wizread(S0_SR, 1, &data);
    if (data == 0) {
      ok = 0;                       // socket did not connect
      return;
    }
  } while (data != 0x17);           // connected
  conout_str(" connected\r\r");
// issue GET
  Wizread_a(S0_TXWR);                   // check tx buf pointer
  pa = TXBASE + (a & TXMASK);           // calc physical buf addr
  if ((pa + getlen) > (TXBASE + TXMASK + 1)){// if tx buf rollover
    i = TXBASE + TXMASK + 1 - pa;
    Wizwrite(pa, i, &get);                   // send upper portion
    Wizwrite(TXBASE, (getlen - i), &get[i]); // send lower portion
  } else Wizwrite(pa, getlen, &get);         // else send all
  a = a + getlen;                       // update tx buf pointer
  Wizwrite_a(S0_TXWR);
  Wizwrite(S0_CR, 1, &SEND_CMD);        // issue send command
  tc = 0;                               // zero total byte count
// receive data
  do {
    Wizread(S0_SR,1,&data);             // wait for webpage
    if (data == 0){
      ok = 0;                           // unexpected socket close
      return;
    }
  } while (data != 0x1c);               // webpage received, server disconnecting
  Wizread_c(S0_RXRSR);
  if (c < 1000) {                       // unexpected response (<1000 bytes)
    ok = 0;
    m = RXBASE;                         // display rxbuffer
    do {
      Wizread(m++,1,&data); conout(data);
    } while (c-- > 0);
  }
//disconnect
  led_off();
  Wizwrite(S0_CR, 1, &CLOSE_CMD);       // close socket
  do {
    Wizread(S0_SR, 1, &data);
  } while (data != 0);
}

Searchweatherpage(void){ // capture desired data, store in txbuff
//first capture and store timestamp
  m = RXBASE;                           // ptr to receive data
  Wizread(m++,1,&data);
  do {
    i=data;
    Wizread(m++,1,&data);
  } while (!((i==0x0a)&&(data=='D')));  // find 'D'ate:
  m += 5;                               // skip 'Date:'
  j = 0;
  do {
    Wizread(m++,1,&data);
    Wizwrite(XMLS+j++,1,&data);         // store timestamp + 0x0d
  } while (data!=0x0a);
  Wizread(m++,1,&data);                 // skip over rest of http header
  do {
    i=data;
    Wizread(m++,1,&data);
  } while (!((i==0x0a)&&(data==0x0d))); // header end = crlfcrlf
// parse the data
  k = 0;                                // ptr into keys
  for (n=0; n < numkeys; n++) {         // for each key
    i = 0;                                //init start key char match count
    do {
      Wizread(m++,1,&data);               // get next byte
      tc++;                               // inc bytes searched count
      if (!(tc % 128)) led_on();          // blink led
      else led_off();
      if (data == keys[k + i]) i++;       // compare to start key char
      else i = 0;
    } while (keys[k + i + 1] != '\b');    // until start key found
    k = k + i + 2;                        // point to finish key
    i = 0;                                // init finish key char match count
    cap = 0;                              // capture off
    do {
      Wizread(m++,1,&data);               // get next byte
      tc++;                               // inc bytes searched count
      if (data == keys[k + i]) i++;       // compare to finish key char
      else i = 0;
      if (data == '<') cap = 0;           // '<' then turn capture off
      if ((cap == 1) && (data != 9) && (data != ' ')){// if capture on
        if (data == 10) data = 13;        // discard blank&tab, swap lf&cr
        Wizwrite(XMLS+j++, 1, &data);     // capture data in txbuff
      }
      if (data == '>') cap = 1;           // '>' then turn capture on
    } while (keys[k + i + 1] != '\b');    // until finish key found
    data = 0;
    Wizwrite(XMLS + j++, 1, &data);       // mark end of each key found data
    k = k + i + 2;                        // point to next key pair
  }                                    // for each key
  led_off();
}

Displayweatherdata(void){ // display keys and captured data
  i = XMLS;                         //ptr to captured data
  do {
    Wizread(i++, 1, &data);         // display timestamp
    conout(data);
  } while (data != 0x0d);

  Wizread_c(S0_RXRSR);
  conout_dec(c);
  conout_str(" bytes received\r");
  conout_dec(tc);
  conout_str(" bytes searched\r");
  conout_dec(j);
  conout_str(" bytes found\r\r");

/*
// display http response header
  m = RXBASE;
  Wizread(m++,1,&data); conout(data);
  do {
    k=data;
    Wizread(m++,1,&data); conout(data);
  } while (!((k==0x0a)&&(data==0x0d))); // header end = crlfcrlf

// display rxbuffer
  for (m=RXBASE; m<RXBASE+c; m++) {
    Wizread(m,1,&data);
    conout(data);
  }
  conout('\r');

// display found data
  for (m=XMLS+31; m<XMLS+j; m++) {
    Wizread(m,1,&data);
    if (data < 0x20) {
        conout('<'); conout_hex(data,2); conout('>');
        if (data == 0) conout ('\r');
    }
    else conout(data);
  }
  conout('\r');
*/

  j = i - XMLS + 1;                      // ptr to xml data
  k = 0;                                 // ptr to keys
  for (m=0; m<numkeys; m++) {            // for each key
    i = 0;
    do {
      conout(keys[k++]);                   // display start key
    } while (keys[k] != '\b');
    conout(' ');
    do {
      Wizread((XMLS + j++), 1, &data);     // display data
      if (data == '\r') {                  // handle special 'nil' values
        Wizread(XMLS + j - 2, 1, &data);   // consecutive cr means 'nil'
        if (data == '\r') conout('0');     // change nil to 0
        data = ' ';
      }
      conout(data);
      Wizread(XMLS + j - 1, 1, &data);
    } while (data != 0);                   // until data separator
    conout(' ');
    k++;
    do {
      conout(keys[k++]);                   // display finish key
    } while (keys[k] != '\b');
    conout('\r');
    k++;
  }                                      // for each key
}


/*********************  Helper Routines   *********************/

Wizread_a(reg){                     // a = 16-bit wizreg
  Wizread(reg, 1, (char *)&a + 1);
  Wizread((reg + 1), 1,(char *)&a);
}

Wizwrite_a(reg){                    // 16-bit wizreg = a
  Wizwrite(reg, 1,(char *)&a + 1);
  Wizwrite((reg + 1), 1,(char *)&a);
}

Wizread_c(reg){                     // c = 16-bit wizreg
  Wizread(reg, 1, (char *)&c + 1);
  Wizread((reg + 1), 1,(char *)&c);
}

Wizread(adr, count, data_ptr){ // single (count=1) or block (count>1) read
  pin_low(nSS);
  shiftout(16, adr);
  shiftout(16, (count & 0x7fff));
  int i;
  for (i=0; i < count; i++) {
    ((char *)data_ptr)[i] = shiftin(8);
  }
  pin_high(nSS);
}

Wizwrite(adr, count, data_ptr){// single (count=1) or block (count>1) write
  pin_low(nSS);
  shiftout(16, adr);
  shiftout(16, (count | 0x8000));
  int i;
  for (i=0; i < count; i++) {
    shiftout(8,((char *)data_ptr)[i]);
  }
  pin_high(nSS);
}

conout_str(char *out){        // out null terminated string
  while (*out != 0) conout(*out++);
}

conout_dec(unsigned int out){ // out 16-bit unsigned decimal
  int sum = 0;
  char temp = '0';
  while (out >= 10000){
    out -= 10000;
    temp++;
    sum++;
  }
  if (sum > 0) conout(temp);
  int i;
  for (i = 0; i < 4; i++){
  temp = '0';
    while (out >= 1000){
      out -= 1000;
      temp++;
      sum++;
    }
    if ((sum > 0) || (i == 3)) conout(temp);
    out *= 10;
  }
}

conout_hex(out, count){      // out count hex digits
  if (count == 0) return;
  long int temp;
  do {
    temp = (out >> ((count-1) << 2)) & 0x0f;
    if (temp < 0x0a) conout(temp + '0');
    else conout((temp - 0x0a) + 'A');
    count--;
  } while (count > 0);
}
