/* Hal for TI Launchpad using UART (usci0) for display */
#include <msp430g2553.h>
#include "pins.h"
// #include <msp430g2452.h>

/****************** Hardware Abstraction ********************/ 

/* modify these routines (and pin assignment in pins.h) for specific MCU

   halinit()            - init hardware (mcu, uart, spi, etc.)
   conout(byte)         - output to debug console
   dir_in(pin)          - set pin direction to input
   dir_out(pin)         - set pin direction to output
   pin_high(pin)        - output pin = 1
   pin_low(pin)         - output pin = 0
   pin_in(pin)          - get input pin, returns 0 or 1
   wait_secs(secs)      - wait for secs seconds
   led_on               - turn led on
   led_off              - turn led off 
   shiftout(count,data) - SPI shiftout count bits from data
   shiftin(count)       - SPI shiftin count bits, return data
*/

// Wiz820io pin assignments
// these pins on msp430 port 1
#define LED 0   //p1.0 led1
#define SCLK 5  //p1.5 ucb0clk
#define MISO 6  //p1.6 ucb0somi
#define MOSI 7  //p1.7 ucb0simo

/* init & mcu-specific hardware */
#define clkrate 16  // set msp430 clock rate = 1 or 16 MHz

halinit(void) {
//clk
	 WDTCTL = WDTPW + WDTHOLD;    // Stop WDT
     DCOCTL = 0;                  // Set clock rate
#if clkrate == 1
     BCSCTL1 = CALBC1_1MHZ;
     DCOCTL = CALDCO_1MHZ;
#endif
#if clkrate == 16
     BCSCTL1 = CALBC1_16MHZ;
     DCOCTL = CALDCO_16MHZ;
#endif

//uart - use MSP430 hardware uart (USCIA0) - 38400 8/n/1
     P1SEL = BIT2;        		  // P1.2=TXD
     P1SEL2 = BIT2;       		  // P1.2=TXD
     UCA0CTL1 |= UCSSEL_2;        // SMCLK
#if clkrate == 1
     UCA0BR0 = 26;                // 1MHz 38400 lsb
     UCA0BR1 = 0;                 // 1MHz 38400 msb
#endif
#if clkrate == 16
	 UCA0BR0 = 0xA0;              // 16MHz 38400 lsb
     UCA0BR1 = 1;                 // 16MHz 38400 msb
     UCA0MCTL = UCBRS1 + UCBRS2;  // Modulation UCBRSx = 6
#endif
     UCA0CTL1 &= ~UCSWRST;        // Start USCI

// led
  dir_out(LED);               // enable led on/off pin
  led_off();                  // init led off

  //spi
/* - This is bit-bang SPI implementation.  To use h/w SPI
   - enable it here and rewrite shiftin & shiftout routines.
*/
  dir_in(MISO);               // using bit-bang spi
  pin_low(MOSI);
  dir_out(MOSI);
  pin_low(SCLK);
  dir_out(SCLK);
}  

/******************   SPI Primitives   ***********************/
/*                Bit-bang SPI interface                     */ 
/*   Shiftout and Shiftin can be rewritten for hardware SPI  */
/*************************************************************/

shiftin(count){
   int spidata = 0;
   pin_low(SCLK);
   pin_low(MOSI);
   int i;
   for (i=0; i < count; i++) {
      spidata = (spidata << 1) | pin_in(MISO);
      pin_high(SCLK);
      pin_low(SCLK);
   }
   return spidata;
}

shiftout(count, spidata){
	pin_low(SCLK);
    spidata = spidata << (16 - count);
    int i;
    for (i=0; i < count; i++) {
      if((spidata & 0x8000) == 0x8000) pin_high(MOSI);
      else pin_low(MOSI);
      spidata = spidata << 1;
      pin_high(SCLK);
      pin_low(SCLK);
    }
}

/* output char to uart (USCIA0) */

conout(char out) {
	  while (!(IFG2 & UCA0TXIFG));        // USCIA0 tx buffer empty?
	  UCA0TXBUF = out;                    // yes, output byte
}

/* pin primitives */

dir_in(int pin) {
  if (pin < 8) P1DIR &= ~(1 << pin);  // set pin direction to input
  else P2DIR &= ~(1 << (pin - 8));
}

dir_out(int pin) {
  if (pin < 8) P1DIR |= 1 << pin;
  else P2DIR |= (1 << (pin - 8));     // set pin direction to output
}

pin_high(int pin) {
  if (pin < 8) P1OUT |= (1 << pin);  // output pin = 1
	  else P2OUT |= (1 << (pin - 8));
}

pin_low(int pin) {
  if (pin < 8) P1OUT &= ~(1 << pin);  // output pin = 0
  else P2OUT &= ~(1 << (pin - 8));
}

pin_in(int pin) {  // get input pin, return 0 or 1
  if (pin < 8) return (P1IN >> pin) & 0x1;
  else return (P2IN >> pin) & 0x1;
}

/* timing primitives */

wait_secs(int secs) {         // wait for 1 second
  do {
#if clkrate == 1
	__delay_cycles(1000000);
#endif
#if clkrate == 4
	__delay_cycles(4000000);
#endif
#if clkrate == 8
	__delay_cycles(8000000);
#endif
#if clkrate == 16
	__delay_cycles(16000000);
#endif
  } while (secs-- > 0);
}
/*
 // sleep for 1 sec
void delay1(){

   IE1 |= WDTIE;                        // Watchdog Interrupt Enable
   WDTCTL = WDTPW+WDTTMSEL+WDTSSEL; // Passwd(WDTPW), Counter-Mode Intervall(WDTTMSEL),Timer auf "0"(WDTCNTCL),SourceClk=ACLCK(WDTSSEL),Sel:00=Clk/32768 01=Clk/8192 10:Clk/512 11=Clk/64

   _BIS_SR(LPM3_bits + GIE);            // put CPU to sleep LPM1_bits

}

// delay interrupt routine, wakes up CPU
#pragma vector=WDT_VECTOR
__interrupt void WATCHDOG_ISR (void){            // interrupt routine for delay
        WDTCTL = WDTPW + WDTHOLD+WDTCNTCL;   //Password(WDTPW), Watchdog stop(WDTHOLD),Counter=0, this resets register(exeption: hold bit)
//        IE1 &= ~WDTIE;                        // Watchdog Interrupt Disable

        __bic_SR_register_on_exit(LPM3_bits); // clear LPM3 bits so the main while loop continues
}
*/

/* led primitives */

led_on(void) {          // turn led on
  pin_high(LED);
}

led_off(void) {        // turn led off
  pin_low(LED);
}
